# Copyright (C) 2005 by Intevation GmbH
# Authors:
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with Thuban for details.

"""
Provide layers via GDAL.
"""

__version__ = "$Revision$"
# $Source$
# $Id$

import os, traceback

from wxPython.wx import wxFileDialog, wxInitAllImageHandlers, wxOPEN, wxID_OK

from Thuban import _
from Thuban.version import versions
from Thuban.Model.resource import has_gdal_support
from Thuban.Model.proj import Projection
from Thuban.Model.extension import Extension
from Thuban.UI.command import registry, Command
from Thuban.UI.mainwindow import main_menu, layer_properties_dialogs
import Thuban.UI.baserenderer

from layer import GDALLayer

if has_gdal_support():
    from gdalwarp import ProjectRasterFile


class GDALExtension(Extension):
    def TreeInfo(self):
        return (_("Extension: %s") % self.title,
                [ object.TreeInfo() for object in self.objects ])

def proj_params_to_str(proj):
    "Build a string suitable for GDAL describing the given projection"
    str = ""
    if proj is not None:
        for p in proj.GetAllParameters():
            str += "+" + p + " "
    return str

def dummy_renderer(renderer, layer):
    pass

def render_gdal_layer(renderer, layer):
    """Project the raster layer using GDAL and draw in by calling
    renderer.draw_raster_data().
    """

    offx, offy = renderer.offset
    width, height = renderer.dc.GetSizeTuple()

    in_proj  = proj_params_to_str(layer.GetProjection())
    out_proj = proj_params_to_str(renderer.map.GetProjection())

    options = 0
    options = options | layer.MaskType()

    xblocks = 2
    yblocks = 2
    xstep=int((width  + (xblocks-1)) / xblocks)
    ystep=int((height + (yblocks-1)) / yblocks)

    for y in range(0, height, ystep):
        for x in range(0, width, xstep):

            w = min(xstep, width-x)
            h = min(ystep, height-y)

            xmin = (x - offx)         / renderer.scale
            xmax = (x+xstep - offx)   / renderer.scale
            ymin = (offy - (y+ystep)) / renderer.scale
            ymax = (offy - y)         / renderer.scale

            img_data = projected_raster_layer(renderer, layer,
                                              in_proj, out_proj,
                                              (xmin, ymin, xmax, ymax),
                                              [0,0], (w, h), options)

            if img_data is not None:
                data = (w, h, img_data)
                renderer.draw_raster_data(layer, x, y, data, "RAW")
                data = None

            yield True

def projected_raster_layer(renderer, layer, srcProj, dstProj, extents,
                           resolution, dimensions, options):
    """Return the projected raster image associated with the layer.

    The returned value will be a tuple of the form

        (image_data, mask_data, alpha_data)

    suitable for the data parameter to draw_raster_data.

    The return value may be None if raster projections are not supported.

    srcProj     --  a string describing the source projection
    dstProj     --  a string describing the destination projection
    extents     --  a tuple of the region to project in map coordinates
    resolution  --  (currently not used, defaults to [0,0])
    dimensions  --  a tuple (width, height) for the output image
    options     --  bit-wise options to pass to the renderer

    the currently supported values for options are

        OPTS_MASK        = 1  -- generate a mask
        OPTS_ALPHA       = 2  -- generate an alpha channel
        OPTS_INVERT_MASK = 4  -- invert the values in the mask 
                                 (if generated)
    """

    ret = None

    if versions['wxPython-tuple'] < (2,5,3):
        options = options | 4  # INVERT_MASK_BITS
        if options & 2:
            options = options | 1 # XXX: fall back to using a mask?
        options = options & ~2 # ALPHA_MASK not supported

    try:
        ret = ProjectRasterFile(layer.GetImageFilename(), 
                                srcProj, dstProj,
                                extents, resolution, dimensions, 
                                options)
    except (MemoryError, IOError, AttributeError, ValueError):
        # Why does this catch AttributeError and ValueError?
        # FIXME: The exception should be communicated to the user
        # better.
        traceback.print_exc()

    return ret


def gdal_dialog(context): 
    mw = context.mainwindow

    dlg = wxFileDialog(mw, _("Select an image file"),
                       mw.application.Path("data"), "", "*.*",
                       wxOPEN)
    if dlg.ShowModal() == wxID_OK:
        filename = dlg.GetPath()
        title = os.path.splitext(os.path.basename(filename))[0]
        map = mw.canvas.Map()
        has_layers = map.HasLayers()
        try:
            layer = GDALLayer(title, filename)
        except IOError:
            # the layer couldn't be opened
            mw.RunMessageBox(_("Add GDAL Image Layer"),
                               _("Can't open the file '%s'.") % filename)
        else:
            map.AddLayer(layer)
            if not has_layers:
                # if we're adding a layer to an empty map, fit the
                # new map to the window
                mw.canvas.FitMapToWindow()
            mw.application.SetPath("data", filename)
    dlg.Destroy()


if has_gdal_support():
    renderer = render_gdal_layer
else:
    renderer = dummy_renderer

Thuban.UI.baserenderer.add_renderer_extension(GDALLayer, renderer)

from properties import GDALProperties
layer_properties_dialogs.add(GDALLayer, GDALProperties)

wxInitAllImageHandlers()
gdal_extension = GDALExtension('GDAL')

map_menu = main_menu.FindOrInsertMenu('map', _('Map'))

registry.Add(Command('gdal', _('Add GDAL Image Layer ...'), gdal_dialog,
                     helptext = _('Add an layer using an image format supported by GDAL')))

map_menu.InsertItem('gdal', after = "rasterlayer_add")
